package org.apache.bcel.generic;

import java.util.Objects;
import org.apache.bcel.classfile.LineNumber;

public class LineNumberGen implements InstructionTargeter, Cloneable
{
    static final LineNumberGen[] EMPTY_ARRAY;
    private InstructionHandle ih;
    private int srcLine;
    
    public LineNumberGen(final InstructionHandle ih, final int srcLine) {
        this.setInstruction(ih);
        this.setSourceLine(srcLine);
    }
    
    public Object clone() {
        try {
            return super.clone();
        }
        catch (final CloneNotSupportedException e) {
            throw new Error("Clone Not Supported");
        }
    }
    
    @Override
    public boolean containsTarget(final InstructionHandle ih) {
        return this.ih == ih;
    }
    
    public InstructionHandle getInstruction() {
        return this.ih;
    }
    
    public LineNumber getLineNumber() {
        return new LineNumber(this.ih.getPosition(), this.srcLine);
    }
    
    public int getSourceLine() {
        return this.srcLine;
    }
    
    public void setInstruction(final InstructionHandle instructionHandle) {
        Objects.requireNonNull(instructionHandle, "instructionHandle");
        BranchInstruction.notifyTarget(this.ih, instructionHandle, this);
        this.ih = instructionHandle;
    }
    
    public void setSourceLine(final int srcLine) {
        this.srcLine = srcLine;
    }
    
    @Override
    public void updateTarget(final InstructionHandle oldIh, final InstructionHandle newIh) {
        if (oldIh != this.ih) {
            throw new ClassGenException("Not targeting " + oldIh + ", but " + this.ih + "}");
        }
        this.setInstruction(newIh);
    }
    
    static {
        EMPTY_ARRAY = new LineNumberGen[0];
    }
}
